from datetime import datetime

from cpl_core.application.application_extension_abc import ApplicationExtensionABC
from cpl_core.configuration import ConfigurationABC
from cpl_core.dependency_injection import ServiceProviderABC
from cpl_core.logging import LoggerABC

from bot_core.configuration.feature_flags_enum import FeatureFlagsEnum
from bot_core.configuration.feature_flags_settings import FeatureFlagsSettings
from bot_core.logging.database_logger import DatabaseLogger
from bot_data.service.migration_service import MigrationService
from bot_data.service.seeder_service import SeederService


class DatabaseExtension(ApplicationExtensionABC):

    def __init__(self):
        pass

    async def run(self, config: ConfigurationABC, services: ServiceProviderABC):
        feature_flags: FeatureFlagsSettings = config.get_configuration(FeatureFlagsSettings)
        if not feature_flags.get_flag(FeatureFlagsEnum.data_module):
            return
        logger: LoggerABC = services.get_service(DatabaseLogger)
        logger.debug(__name__, 'Database extension started')
        config.add_configuration('Database_StartTime', str(datetime.now()))
        migrations: MigrationService = services.get_service(MigrationService)
        migrations.migrate()
