import asyncio
import time
import unittest
from typing import Optional

from cpl_core.configuration import ConfigurationABC
from cpl_core.dependency_injection import ServiceProviderABC

from ui_tests.main import get_app
from ui_tests_shared.configuration.test_settings import TestSettings
from ui_tests_shared.declarations import Declarations
from ui_tests_shared.ui import UI


class TestCaseWithApp(unittest.TestCase):
    _config: Optional[ConfigurationABC] = None
    _services: Optional[ServiceProviderABC] = None
    _test_settings: Optional[TestSettings] = None

    @classmethod
    def setUpClass(cls):
        if Declarations.app is None:
            app = get_app()
            import nest_asyncio

            nest_asyncio.apply()
            asyncio.run(app.run_async())

        cls._config = Declarations.app.config
        cls._services = Declarations.app.services
        cls._test_settings: TestSettings = cls._config.get_configuration(TestSettings)
        UI.set_settings(cls._test_settings)
        UI.login(cls._config.get_configuration('DISCORD_MAIL'), cls._config.get_configuration('DISCORD_PASSWORD'))

    @classmethod
    def tearDownClass(cls):
        if cls._config.get_configuration('IS_UNITTEST'):
            loop = asyncio.get_event_loop()
            loop.run_until_complete(Declarations.app.stop_async())
